@@ -19,11 +19,10 @@
 #include <QTest>
 #include <QLocale>
 #include <QJsonDocument>
-#include <QFileInfo>
-#include <QJsonArray>
+#include <QDebug>
 
-#include <KAboutData>
 #include <KPluginMetaData>
+#include <KAboutData>
 
 #include <kplugininfo.h>
 #include <kservice.h>
@@ -84,13 +83,17 @@ private Q_SLOTS:
         // for most constructors translations are performed when the object is constructed and not at runtime!
         QLocale::setDefault(QLocale::c());
         KPluginInfo info(fakepluginDesktop);
+#if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 0)
         KService::Ptr fakepluginService(new KService(fakepluginDesktop));
         KPluginInfo infoFromService(fakepluginService);
+#endif
         KPluginInfo compatJsonInfo(KPluginMetaData(compatJson, pluginName));
         QLocale::setDefault(QLocale(QLocale::German, QLocale::Germany));
         KPluginInfo infoGerman(fakepluginDesktop);
+#if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 0)
         KService::Ptr fakepluginServiceGerman(new KService(fakepluginDesktop));
         KPluginInfo infoFromServiceGerman(fakepluginServiceGerman);
+#endif
         KPluginInfo compatJsonInfoGerman(KPluginMetaData(compatJson, pluginName));
         QLocale::setDefault(QLocale::c());
 
@@ -100,11 +103,13 @@ private Q_SLOTS:
         QTest::newRow("from .desktop") << fakepluginDesktop << info << infoGerman << QVariant() << false;
         QTest::newRow("with custom property") << info.libraryPath() << withCustomProperty(info)
             << withCustomProperty(infoGerman) << QVariant("Baz") << false;
+#if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 0)
         QTest::newRow("from KService::Ptr") << fakepluginDesktop << infoFromService
                 << infoFromServiceGerman << QVariant() << true;
         QTest::newRow("from KService::Ptr + custom property") << pluginName
                 << withCustomProperty(infoFromService) << withCustomProperty(infoFromServiceGerman)
                 << QVariant("Baz") << true;
+#endif
         QTest::newRow("from JSON file") << pluginName << jsonInfo << jsonInfo << QVariant() << false;
         QTest::newRow("from JSON file + custom property") << pluginName
                 << withCustomProperty(jsonInfo) << withCustomProperty(jsonInfo) << QVariant("Baz") << false;
@@ -146,8 +151,11 @@ private Q_SLOTS:
             QCOMPARE(infoGerman.name(), QStringLiteral("NSA-Modul"));
         }
 
-        QCOMPARE(info.author(), QStringLiteral("Sebastian Kügler"));
+        QCOMPARE(info.author(), QString::fromUtf8("Sebastian Kügler"));
         QCOMPARE(info.category(), QStringLiteral("Examples"));
+        if (!info.dependencies().isEmpty()) {
+            qWarning() << "Got dependencies" << info.dependencies();
+        }
         QCOMPARE(info.dependencies(), QStringList());
         QCOMPARE(info.email(), QStringLiteral("sebas@kde.org"));
         QCOMPARE(info.entryPath(), desktopFilePath);
@@ -197,7 +205,7 @@ private Q_SLOTS:
 
 
         QCOMPARE(meta.authors().size(), 1);
-        QCOMPARE(meta.authors().at(0).name(), QStringLiteral("Sebastian Kügler"));
+        QCOMPARE(meta.authors().at(0).name(), QString::fromUtf8("Sebastian Kügler"));
         QCOMPARE(meta.authors().at(0).emailAddress(), QStringLiteral("sebas@kde.org"));
         QCOMPARE(meta.category(), QStringLiteral("Examples"));
         QCOMPARE(meta.dependencies(), QStringList());
@@ -270,7 +278,7 @@ private Q_SLOTS:
         QCOMPARE(infoGerman.name(), QStringLiteral("NSA-Modul"));
         QLocale::setDefault(QLocale::c());
 
-        QCOMPARE(info.author(), QStringLiteral("Sebastian Kügler"));
+        QCOMPARE(info.author(), QString::fromUtf8("Sebastian Kügler"));
         QCOMPARE(info.category(), QStringLiteral("Examples"));
         QCOMPARE(info.dependencies(), QStringList());
         QCOMPARE(info.email(), QStringLiteral("sebas@kde.org"));
