@@ -44,6 +44,10 @@
 #define Q_XDG_PLATFORM
 #endif
 
+// On Unix, lastModified() finally returns milliseconds as well, since Qt 5.8.0
+// Not sure about the situation on Windows though.
+static const int s_waitDelay = 10;
+
 extern KSERVICE_EXPORT int ksycoca_ms_between_checks;
 
 class KSycocaTest : public QObject
@@ -53,7 +57,7 @@ class KSycocaTest : public QObject
 private Q_SLOTS:
     void initTestCase()
     {
-        QStandardPaths::enableTestMode(true);
+        QStandardPaths::setTestModeEnabled(true);
 
         QVERIFY(m_tempDir.isValid());
 
@@ -84,9 +88,9 @@ private Q_SLOTS:
     void kBuildSycocaShouldEmitDatabaseChanged();
     void dirInFutureShouldRebuildSycocaOnce();
     void dirTimestampShouldBeCheckedRecursively();
+    void recursiveCheckShouldIgnoreLinksGoingUp();
     void testAllResourceDirs();
     void testDeletingSycoca();
-    void testGlobalSycoca();
     void testNonReadableSycoca();
 
 private:
@@ -100,9 +104,10 @@ private:
         file.sync();
         qDebug() << "created" << serviceTypesDir() + "/fakeGlobalServiceType.desktop";
     }
-    QString servicesDir() { return QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + "/kservices5"; }
-    QString serviceTypesDir() { return QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + "/kservicetypes5"; }
-    QString menusDir() { return QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation) + "/menus"; }
+    QString servicesDir() const { return QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + "/kservices5"; }
+    QString serviceTypesDir() const { return QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + "/kservicetypes5"; }
+    QString menusDir() const { return QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation) + "/menus"; }
+    QString appsDir() const { return QStandardPaths::writableLocation(QStandardPaths::ApplicationsLocation) + QLatin1Char('/'); }
 
     static void runKBuildSycoca(const QProcessEnvironment &environment, bool global = false);
 
@@ -122,7 +127,7 @@ void KSycocaTest::ensureCacheValidShouldCreateDB() // this is what kded does on
 void KSycocaTest::kBuildSycocaShouldEmitDatabaseChanged()
 {
     // It used to be a DBus signal, now it's file watching
-    QTest::qWait(1000); // ensure the file watching notices it's a new second
+    QTest::qWait(s_waitDelay);
     // Ensure kbuildsycoca has something to do
     QVERIFY(QFile::remove(serviceTypesDir() + "/fakeGlobalServiceType.desktop"));
     // Run kbuildsycoca
@@ -143,7 +148,7 @@ void KSycocaTest::dirInFutureShouldRebuildSycocaOnce()
 #ifdef Q_OS_UNIX
     const QString path = serviceTypesDir();
     struct timeval tp;
-    gettimeofday(&tp, 0);
+    gettimeofday(&tp, nullptr);
     struct utimbuf utbuf;
     utbuf.actime = tp.tv_sec;
     utbuf.modtime = tp.tv_sec + 60; // 60 second in the future
@@ -155,13 +160,13 @@ void KSycocaTest::dirInFutureShouldRebuildSycocaOnce()
 #endif
     ksycoca_ms_between_checks = 0;
 
-    QTest::qWait(1000); // remove this once lastModified includes ms
+    QTest::qWait(s_waitDelay);
 
     KSycoca::self()->ensureCacheValid();
     const QDateTime newTimestamp = QFileInfo(KSycoca::absoluteFilePath()).lastModified();
     QVERIFY(newTimestamp > oldTimestamp);
 
-    QTest::qWait(1000); // remove this once lastModified includes ms
+    QTest::qWait(s_waitDelay);
 
     KSycoca::self()->ensureCacheValid();
     const QDateTime againTimestamp = QFileInfo(KSycoca::absoluteFilePath()).lastModified();
@@ -188,7 +193,7 @@ void KSycocaTest::dirTimestampShouldBeCheckedRecursively()
     // ### use QFile::setFileTime when it lands in Qt...
 #ifdef Q_OS_UNIX
     struct timeval tp;
-    gettimeofday(&tp, 0);
+    gettimeofday(&tp, nullptr);
     struct utimbuf utbuf;
     utbuf.actime = tp.tv_sec;
     utbuf.modtime = tp.tv_sec + 60; // 60 second in the future
@@ -198,7 +203,7 @@ void KSycocaTest::dirTimestampShouldBeCheckedRecursively()
 #endif
 
     ksycoca_ms_between_checks = 0;
-    QTest::qWait(1000); // remove this once lastModified includes ms
+    QTest::qWait(s_waitDelay);
 
     qDebug() << "Waited 1s, calling ensureCacheValid (should rebuild)";
     KSycoca::self()->ensureCacheValid();
@@ -208,7 +213,7 @@ void KSycocaTest::dirTimestampShouldBeCheckedRecursively()
     }
     QVERIFY(newTimestamp > oldTimestamp);
 
-    QTest::qWait(1000); // remove this once lastModified includes ms
+    QTest::qWait(s_waitDelay);
 
     qDebug() << "Waited 1s, calling ensureCacheValid (should not rebuild)";
     KSycoca::self()->ensureCacheValid();
@@ -219,6 +224,46 @@ void KSycocaTest::dirTimestampShouldBeCheckedRecursively()
     QDir(path).removeRecursively();
 }
 
+void KSycocaTest::recursiveCheckShouldIgnoreLinksGoingUp()
+{
+#ifndef Q_OS_UNIX
+    QSKIP("This test requires symlinks and utime");
+#endif
+    ksycoca_ms_between_checks = 0;
+    const QString link = menusDir() + QLatin1String("/linkGoingUp");
+    QVERIFY(QFile::link("..", link));
+    QTest::qWait(s_waitDelay);
+    KSycoca::self()->ensureCacheValid();
+    QVERIFY2(QFile::exists(KSycoca::absoluteFilePath()), qPrintable(KSycoca::absoluteFilePath()));
+    const QDateTime oldTimestamp = QFileInfo(KSycoca::absoluteFilePath()).lastModified();
+    QVERIFY(oldTimestamp.isValid());
+
+    const QString path = QFileInfo(menusDir()).absolutePath(); // the parent of the menus dir
+
+    // ### use QFile::setFileTime when it lands in Qt...
+#ifdef Q_OS_UNIX
+    struct timeval tp;
+    gettimeofday(&tp, nullptr);
+    struct utimbuf utbuf;
+    utbuf.actime = tp.tv_sec;
+    utbuf.modtime = tp.tv_sec + 60; // 60 second in the future
+    QCOMPARE(utime(QFile::encodeName(path).constData(), &utbuf), 0);
+    qDebug("Time changed for %s", qPrintable(path));
+    qDebug() << QDateTime::currentDateTime() << QFileInfo(path).lastModified();
+#endif
+
+    ksycoca_ms_between_checks = 0;
+    QTest::qWait(s_waitDelay);
+
+    qDebug() << "Waited 1s, calling ensureCacheValid (should not rebuild)";
+    KSycoca::self()->ensureCacheValid();
+    const QDateTime againTimestamp = QFileInfo(KSycoca::absoluteFilePath()).lastModified();
+    QCOMPARE(againTimestamp, oldTimestamp); // same mtime, it didn't get rebuilt
+
+    // Ensure we don't pollute the other tests
+    QFile(link).remove();
+}
+
 
 void KSycocaTest::runKBuildSycoca(const QProcessEnvironment &environment, bool global)
 {
@@ -256,37 +301,6 @@ void KSycocaTest::testDeletingSycoca()
     QVERIFY(QFile::exists(KSycoca::absoluteFilePath()));
 }
 
-void KSycocaTest::testGlobalSycoca()
-{
-    // No local DB
-    QFile::remove(KSycoca::absoluteFilePath());
-
-    // Build global DB
-    // We could do it in-process, but let's check what a sysadmin would do: run kbuildsycoca5 --global
-    QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
-    env.insert(QStringLiteral("XDG_DATA_DIRS"), m_tempDir.path());
-    runKBuildSycoca(env, true /*global*/);
-    QVERIFY(QFile::exists(KSycoca::absoluteFilePath(KSycoca::GlobalDatabase)));
-
-    KSycoca::self()->ensureCacheValid();
-    QVERIFY(!QFile::exists(KSycoca::absoluteFilePath()));
-
-    // Now create a local file, after a 1s delay, until QDateTime includes ms...
-    QTest::qWait(1000);
-    KDesktopFile file(serviceTypesDir() + "/fakeLocalServiceType.desktop");
-    KConfigGroup group = file.desktopGroup();
-    group.writeEntry("Comment", "Fake Local ServiceType");
-    group.writeEntry("Type", "ServiceType");
-    group.writeEntry("X-KDE-ServiceType", "FakeLocalServiceType");
-    file.sync();
-    //qDebug() << "created" << serviceTypesDir() + "/fakeLocalServiceType.desktop at" << QDateTime::currentMSecsSinceEpoch();
-
-    // Using ksycoca should now build a local one
-    ksycoca_ms_between_checks = 0;
-    QVERIFY(KServiceType::serviceType(QStringLiteral("FakeLocalServiceType")));
-    QVERIFY(QFile::exists(KSycoca::absoluteFilePath()));
-}
-
 void KSycocaTest::testNonReadableSycoca()
 {
     // Lose readability (to simulate e.g. owned by root)
@@ -294,7 +308,7 @@ void KSycocaTest::testNonReadableSycoca()
     ksycoca_ms_between_checks = 0;
     KBuildSycoca builder;
     QVERIFY(builder.recreate());
-    QVERIFY(KServiceType::serviceType(QStringLiteral("FakeGlobalServiceType")));
+    QVERIFY(!KServiceType::serviceType(QStringLiteral("FakeGlobalServiceType")));
 
     // cleanup
     QFile::remove(KSycoca::absoluteFilePath());
