@@ -23,7 +23,8 @@
 #include "kconfiggroup.h"
 
 #include <QCoreApplication>
-#include <QtCore/QFile>
+#include <QDir>
+#include <QFile>
 #include <QStringList>
 
 class KAutostartPrivate
@@ -72,20 +73,25 @@ KAutostart::KAutostart(const QString &entryName, QObject *parent)
     : QObject(parent),
       d(new KAutostartPrivate)
 {
-    if (entryName.isEmpty()) {
-        d->name = QCoreApplication::applicationName();
+    const bool isAbsolute = QDir::isAbsolutePath(entryName);
+    if (isAbsolute) {
+        d->name = entryName.mid(entryName.lastIndexOf(QLatin1Char('/')) + 1);
     } else {
-        d->name = entryName;
+        if (entryName.isEmpty()) {
+            d->name = QCoreApplication::applicationName();
+        } else {
+            d->name = entryName;
+        }
+
+        if (!d->name.endsWith(QLatin1String(".desktop"))) {
+            d->name.append(QLatin1String(".desktop"));
+        }
     }
 
-    if (!d->name.endsWith(QLatin1String(".desktop"))) {
-        d->name.append(QStringLiteral(".desktop"));
-    }
-
-    const QString path = QStandardPaths::locate(QStandardPaths::GenericConfigLocation, QStringLiteral("autostart/") + d->name);
+    const QString path = isAbsolute ? entryName : QStandardPaths::locate(QStandardPaths::GenericConfigLocation, QLatin1String("autostart/") + d->name);
     if (path.isEmpty()) {
         // just a new KDesktopFile, since we have nothing to use
-        d->df = new KDesktopFile(QStandardPaths::GenericConfigLocation, QStringLiteral("autostart/") + d->name);
+        d->df = new KDesktopFile(QStandardPaths::GenericConfigLocation, QLatin1String("autostart/") + d->name);
         d->copyIfNeededChecked = true;
     } else {
         d->df = new KDesktopFile(path);
@@ -202,7 +208,7 @@ void KAutostart::setVisibleName(const QString &name)
 bool KAutostart::isServiceRegistered(const QString &entryName)
 {
     const QString localDir = QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation) + QLatin1String("/autostart/");
-    return QFile::exists(localDir + entryName + QStringLiteral(".desktop"));
+    return QFile::exists(localDir + entryName + QLatin1String(".desktop"));
 }
 
 QString KAutostart::commandToCheck() const
@@ -222,7 +228,7 @@ void KAutostart::setCommandToCheck(const QString &exec)
 
 // do not specialize the readEntry template -
 // http://connect.microsoft.com/VisualStudio/feedback/ViewFeedback.aspx?FeedbackID=100911
-static KAutostart::StartPhase readEntry(const KConfigGroup &group, const char *key, const KAutostart::StartPhase &aDefault)
+static KAutostart::StartPhase readEntry(const KConfigGroup &group, const char *key, KAutostart::StartPhase aDefault)
 {
     const QByteArray data = group.readEntry(key, QByteArray());
 
