@@ -26,7 +26,6 @@
 #include <qplatformdefs.h>
 
 #include <QCharRef>
-#include <QFile>
 #include <QDir>
 #include <QMap>
 #include <QCoreApplication>
@@ -160,7 +159,7 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
     entryMap.remove(QStringLiteral("Terminal"));
     m_strTerminalOptions = desktopGroup.readEntry("TerminalOptions");   // should be a property IMHO
     entryMap.remove(QStringLiteral("TerminalOptions"));
-    m_strPath = config->readPath();
+    m_strWorkingDirectory = config->readPath();
     entryMap.remove(QStringLiteral("Path"));
     m_strComment = config->readComment();
     entryMap.remove(QStringLiteral("Comment"));
@@ -255,7 +254,7 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
     if (entry != entryMap.end()) {
         const QString &key = entry.key();
         m_mapProps.insert(key, QVariant(desktopGroup.readEntryUntranslated(key)));
-        m_mapProps.insert(key + QStringLiteral("-Translated"), QVariant(*entry));
+        m_mapProps.insert(key + QLatin1String("-Translated"), QVariant(*entry));
         entryMap.erase(entry);
     }
 
@@ -318,7 +317,7 @@ void KServicePrivate::load(QDataStream &s)
     // number in ksycoca.cpp
     s >> m_strType >> m_strName >> m_strExec >> m_strIcon
       >> term >> m_strTerminalOptions
-      >> m_strPath >> m_strComment >> def >> m_mapProps
+      >> m_strWorkingDirectory >> m_strComment >> def >> m_mapProps
       >> m_strLibrary
       >> dst
       >> m_strDesktopEntryName
@@ -348,7 +347,7 @@ void KServicePrivate::save(QDataStream &s)
     // number in ksycoca.cpp
     s << m_strType << m_strName << m_strExec << m_strIcon
       << term << m_strTerminalOptions
-      << m_strPath << m_strComment << def << m_mapProps
+      << m_strWorkingDirectory << m_strComment << def << m_mapProps
       << m_strLibrary
       << dst
       << m_strDesktopEntryName
@@ -395,6 +394,11 @@ KService::KService(QDataStream &_str, int _offset)
 {
 }
 
+KService::KService(const KService &other)
+   : KSycocaEntry(*new KServicePrivate(*static_cast<KServicePrivate *>(other.d_ptr)))
+{
+}
+
 KService::~KService()
 {
 }
@@ -510,7 +514,7 @@ QVariant KServicePrivate::property(const QString &_name, QVariant::Type t) const
     } else if (_name == QLatin1String("TerminalOptions")) {
         return makeStringVariant(m_strTerminalOptions);
     } else if (_name == QLatin1String("Path")) {
-        return makeStringVariant(m_strPath);
+        return makeStringVariant(m_strWorkingDirectory);
     } else if (_name == QLatin1String("Comment")) {
         return makeStringVariant(m_strComment);
     } else if (_name == QLatin1String("GenericName")) {
@@ -651,11 +655,17 @@ bool KService::showInCurrentDesktop() const
 {
     Q_D(const KService);
 
-    QStringList currentDesktops(QString::fromLatin1(qgetenv("XDG_CURRENT_DESKTOP")));
+    const QString envVar = QString::fromLatin1(qgetenv("XDG_CURRENT_DESKTOP"));
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
+    QVector<QStringRef> currentDesktops(envVar.splitRef(QLatin1Char(':'), QString::SkipEmptyParts));
+#else
+    QVector<QStringRef> currentDesktops(envVar.splitRef(QLatin1Char(':'), Qt::SkipEmptyParts));
+#endif
+    const QString kde = QStringLiteral("KDE");
     if (currentDesktops.isEmpty()) {
         // This could be an old display manager, or e.g. a failsafe session with no desktop name
         // In doubt, let's say we show KDE stuff.
-        currentDesktops << QStringLiteral("KDE");
+        currentDesktops.append(&kde);
     }
 
     // This algorithm is described in the desktop entry spec
@@ -663,7 +673,7 @@ bool KService::showInCurrentDesktop() const
     QMap<QString, QVariant>::ConstIterator it = d->m_mapProps.find(QStringLiteral("OnlyShowIn"));
     if ((it != d->m_mapProps.end()) && (it->isValid())) {
         const QStringList aList = it->toString().split(QLatin1Char(';'));
-        foreach (const QString &desktop, currentDesktops) {
+        for (const QStringRef &desktop : qAsConst(currentDesktops)) {
             if (aList.contains(desktop)) {
                 return true;
             }
@@ -674,7 +684,7 @@ bool KService::showInCurrentDesktop() const
     it = d->m_mapProps.find(QStringLiteral("NotShowIn"));
     if ((it != d->m_mapProps.end()) && (it->isValid())) {
         const QStringList aList = it->toString().split(QLatin1Char(';'));
-        foreach (const QString &desktop, currentDesktops) {
+        for (const QStringRef &desktop : qAsConst(currentDesktops)) {
             if (aList.contains(desktop)) {
                 return false;
             }
@@ -911,10 +921,18 @@ KService::DBusStartupType KService::dbusStartupType() const
     return d->m_DBUSStartusType;
 }
 
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 63)
 QString KService::path() const
 {
     Q_D(const KService);
-    return d->m_strPath;
+    return d->m_strWorkingDirectory;
+}
+#endif
+
+QString KService::workingDirectory() const
+{
+    Q_D(const KService);
+    return d->m_strWorkingDirectory;
 }
 
 QString KService::comment() const
@@ -967,11 +985,13 @@ QStringList KService::mimeTypes() const
     return ret;
 }
 
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 67)
 bool KService::allowAsDefault() const
 {
     Q_D(const KService);
     return d->m_bAllowAsDefault;
 }
+#endif
 
 int KService::initialPreference() const
 {
