@@ -22,9 +22,9 @@
 #include <ksycocatype.h>
 #include <kservice_export.h>
 
-#include <QtCore/QDataStream>
-#include <QtCore/QStringList>
-#include <QtCore/QVariant>
+#include <QDataStream>
+#include <QStringList>
+#include <QVariant>
 #include <QExplicitlySharedDataPointer>
 
 class KSycocaEntryPrivate;
@@ -125,6 +125,7 @@ private:
     // All these need access to offset()
     friend class KSycocaFactory;
     friend class KBuildServiceFactory;
+    friend class KServiceFactory;
     friend class KMimeTypeTrader;
     friend class KServiceTypeTrader;
     friend class KService;
