@@ -24,11 +24,10 @@
 #include <kservice.h>
 
 #include <QDebug>
-#include <QtCore/QMap>
-#include <QtCore/QFile>
-#include <QtCore/QDir>
-#include <QtCore/QRegExp>
-#include <QtCore/QDirIterator>
+#include <QMap>
+#include <QFile>
+#include <QDir>
+#include <QDirIterator>
 #include <QStandardPaths>
 
 static void foldNode(QDomElement &docElem, QDomElement &e, QMap<QString, QDomElement> &dupeList, QString s = QString()) //krazy:exclude=passbyvalue
@@ -38,7 +37,7 @@ static void foldNode(QDomElement &docElem, QDomElement &e, QMap<QString, QDomEle
     }
     QMap<QString, QDomElement>::iterator it = dupeList.find(s);
     if (it != dupeList.end()) {
-        //qDebug() << e.tagName() << "and" << s << "requires combining!";
+        //qCDebug(SYCOCA) << e.tagName() << "and" << s << "requires combining!";
 
         docElem.removeChild(*it);
         dupeList.erase(it);
@@ -59,12 +58,12 @@ static void replaceNode(QDomElement &docElem, QDomNode &n, const QStringList &li
     QDomNode next = n.nextSibling();
     docElem.removeChild(n);
     n = next;
-//   qDebug() << "Next tag = " << n.toElement().tagName();
+//   qCDebug(SYCOCA) << "Next tag = " << n.toElement().tagName();
 }
 
 void VFolderMenu::registerFile(const QString &file)
 {
-    int i = file.lastIndexOf('/');
+    int i = file.lastIndexOf(QLatin1Char('/'));
     if (i < 0) {
         return;
     }
@@ -114,7 +113,7 @@ track(const QString &menuId, const QString &menuName, const QHash<QString, KServ
 void
 VFolderMenu::includeItems(QHash<QString, KService::Ptr> &items1, const QHash<QString, KService::Ptr> &items2)
 {
-    foreach (const KService::Ptr &p, items2) {
+    for (const KService::Ptr &p : items2) {
         items1.insert(p->menuId(), p);
     }
 }
@@ -122,7 +121,8 @@ VFolderMenu::includeItems(QHash<QString, KService::Ptr> &items1, const QHash<QSt
 void
 VFolderMenu::matchItems(QHash<QString, KService::Ptr> &items1, const QHash<QString, KService::Ptr> &items2)
 {
-    foreach (const KService::Ptr &p, items1) {
+    const QHash<QString, KService::Ptr> tmpItems1 = items1;
+    for (const KService::Ptr &p : tmpItems1) {
         QString id = p->menuId();
         if (!items2.contains(id)) {
             items1.remove(id);
@@ -133,7 +133,7 @@ VFolderMenu::matchItems(QHash<QString, KService::Ptr> &items1, const QHash<QStri
 void
 VFolderMenu::excludeItems(QHash<QString, KService::Ptr> &items1, const QHash<QString, KService::Ptr> &items2)
 {
-    foreach (const KService::Ptr &p, items2) {
+    for (const KService::Ptr &p : items2) {
         items1.remove(p->menuId());
     }
 }
@@ -141,7 +141,7 @@ VFolderMenu::excludeItems(QHash<QString, KService::Ptr> &items1, const QHash<QSt
 VFolderMenu::SubMenu *
 VFolderMenu::takeSubMenu(SubMenu *parentMenu, const QString &menuName)
 {
-    const int i = menuName.indexOf('/');
+    const int i = menuName.indexOf(QLatin1Char('/'));
     const QString s1 = i > 0 ? menuName.left(i) : menuName;
     const QString s2 = menuName.mid(i + 1);
 
@@ -221,12 +221,12 @@ VFolderMenu::mergeMenu(SubMenu *menu1, SubMenu *menu2, bool reversePriority)
 void
 VFolderMenu::insertSubMenu(SubMenu *parentMenu, const QString &menuName, SubMenu *newMenu, bool reversePriority)
 {
-    const int i = menuName.indexOf('/');
+    const int i = menuName.indexOf(QLatin1Char('/'));
     const QString s1 = menuName.left(i);
     const QString s2 = menuName.mid(i + 1);
 
     // Look up menu
-    foreach (SubMenu *menu, parentMenu->subMenus) {
+    for (SubMenu *menu : qAsConst(parentMenu->subMenus)) {
         if (menu->name == s1) {
             if (i == -1) {
                 mergeMenu(menu, newMenu, reversePriority);
@@ -252,7 +252,7 @@ VFolderMenu::insertSubMenu(SubMenu *parentMenu, const QString &menuName, SubMenu
 void
 VFolderMenu::insertService(SubMenu *parentMenu, const QString &name, KService::Ptr newService)
 {
-    const int i = name.indexOf('/');
+    const int i = name.indexOf(QLatin1Char('/'));
 
     if (i == -1) {
         // Add it here
@@ -264,7 +264,7 @@ VFolderMenu::insertService(SubMenu *parentMenu, const QString &name, KService::P
     QString s2 = name.mid(i + 1);
 
     // Look up menu
-    foreach (SubMenu *menu, parentMenu->subMenus) {
+    for (SubMenu *menu : qAsConst(parentMenu->subMenus)) {
         if (menu->name == s1) {
             insertService(menu, s2, newService);
             return;
@@ -296,7 +296,7 @@ VFolderMenu::~VFolderMenu()
 }
 
 #define FOR_ALL_APPLICATIONS(it) \
-    foreach (AppsInfo *info, m_appsInfoStack) \
+    for (AppsInfo *info : qAsConst(m_appsInfoStack)) \
     { \
         QHashIterator<QString,KService::Ptr> it = info->applications; \
         while (it.hasNext()) \
@@ -305,7 +305,7 @@ VFolderMenu::~VFolderMenu()
 #define FOR_ALL_APPLICATIONS_END } }
 
 #define FOR_CATEGORY(category, it) \
-    foreach (AppsInfo *info, m_appsInfoStack) \
+    for (AppsInfo *info : qAsConst(m_appsInfoStack)) \
     { \
         const KService::List list = info->dictCategories.value(category); \
         for(KService::List::ConstIterator it = list.constBegin(); \
@@ -316,7 +316,7 @@ VFolderMenu::~VFolderMenu()
 KService::Ptr
 VFolderMenu::findApplication(const QString &relPath)
 {
-    foreach (AppsInfo *info, m_appsInfoStack) {
+    for (AppsInfo *info : qAsConst(m_appsInfoStack)) {
         if (info->applications.contains(relPath)) {
             KService::Ptr s = info->applications[relPath];
             if (s) {
@@ -338,7 +338,7 @@ VFolderMenu::addApplication(const QString &id, KService::Ptr service)
 void
 VFolderMenu::buildApplicationIndex(bool unusedOnly)
 {
-    foreach (AppsInfo *info, m_appsInfoList) {
+    for (AppsInfo *info : qAsConst(m_appsInfoList)) {
         info->dictCategories.clear();
         QMutableHashIterator<QString, KService::Ptr> it = info->applications;
         while (it.hasNext()) {
@@ -349,7 +349,8 @@ VFolderMenu::buildApplicationIndex(bool unusedOnly)
                 continue;
             }
 
-            Q_FOREACH (const QString &cat, s->categories()) {
+            const auto categories = s->categories();
+            for (const QString &cat : categories) {
                 info->dictCategories[cat].append(s); // find or insert entry in hash
             }
         }
@@ -411,7 +412,7 @@ VFolderMenu::absoluteDir(const QString &_dir, const QString &baseDir, bool keepR
     bool relative = QDir::isRelativePath(dir);
     if (relative && !keepRelativeToCfg) {
         relative = false;
-        dir = QStandardPaths::locate(QStandardPaths::GenericConfigLocation, QStringLiteral("menus/") + dir, QStandardPaths::LocateDirectory);
+        dir = QStandardPaths::locate(QStandardPaths::GenericConfigLocation, QLatin1String("menus/") + dir, QStandardPaths::LocateDirectory);
     }
 
     if (!relative) {
@@ -421,8 +422,8 @@ VFolderMenu::absoluteDir(const QString &_dir, const QString &baseDir, bool keepR
         }
     }
 
-    if (!dir.endsWith('/')) {
-        dir += '/';
+    if (!dir.endsWith(QLatin1Char('/'))) {
+        dir += QLatin1Char('/');
     }
 
     return dir;
@@ -483,7 +484,7 @@ VFolderMenu::loadDoc()
 void
 VFolderMenu::mergeFile(QDomElement &parent, const QDomNode &mergeHere)
 {
-    //qDebug() << m_docInfo.path;
+    //qCDebug(SYCOCA) << m_docInfo.path;
     QDomDocument doc = loadDoc();
 
     QDomElement docElem = doc.documentElement();
@@ -497,7 +498,7 @@ VFolderMenu::mergeFile(QDomElement &parent, const QDomNode &mergeHere)
             // Skip
         }
         // The spec says we must ignore any Name nodes
-        else if (e.tagName() != "Name") {
+        else if (e.tagName() != QLatin1String("Name")) {
             parent.insertAfter(n, last);
             last = n;
         }
@@ -522,7 +523,7 @@ VFolderMenu::mergeMenus(QDomElement &docElem, QString &name)
     while (!n.isNull()) {
         QDomElement e = n.toElement(); // try to convert the node to an element.
         if (e.isNull()) {
-// qDebug() << "Empty node";
+// qCDebug(SYCOCA) << "Empty node";
         } else if (e.tagName() == QLatin1String("DefaultAppDirs")) {
             // Replace with m_defaultAppDirs
             replaceNode(docElem, n, m_defaultAppDirs, QStringLiteral("AppDir"));
@@ -601,18 +602,18 @@ VFolderMenu::mergeMenus(QDomElement &docElem, QString &name)
             continue;
         } else if (e.tagName() == QLatin1String("MergeDir")) {
             QString dir = absoluteDir(e.text(), e.attribute(QStringLiteral("__BaseDir")), true);
-            Q_ASSERT(dir.endsWith('/'));
+            Q_ASSERT(dir.endsWith(QLatin1Char('/')));
 
             const bool relative = QDir::isRelativePath(dir);
-            const QStringList dirs = QStandardPaths::locateAll(QStandardPaths::GenericConfigLocation, QStringLiteral("menus/") + dir, QStandardPaths::LocateDirectory);
-            Q_FOREACH (const QString &menuDir, dirs) {
+            const QStringList dirs = QStandardPaths::locateAll(QStandardPaths::GenericConfigLocation, QLatin1String("menus/") + dir, QStandardPaths::LocateDirectory);
+            for (const QString &menuDir : dirs) {
                 registerDirectory(menuDir);
             }
 
             QStringList fileList;
-            Q_FOREACH (const QString &menuDir, dirs) {
+            for (const QString &menuDir : dirs) {
                 const QStringList fileNames = QDir(menuDir).entryList(QStringList() << QStringLiteral("*.menu"));
-                Q_FOREACH (const QString &file, fileNames) {
+                for (const QString &file : fileNames) {
                     const QString fileToAdd = relative ? dir + file : menuDir + file;
                     if (!fileList.contains(fileToAdd)) {
                         fileList.append(fileToAdd);
@@ -620,7 +621,7 @@ VFolderMenu::mergeMenus(QDomElement &docElem, QString &name)
                 }
             }
 
-            Q_FOREACH (const QString &file, fileList) {
+            for (const QString &file : qAsConst(fileList)) {
                 pushDocInfo(file);
                 mergeFile(docElem, n);
                 popDocInfo();
@@ -651,7 +652,8 @@ VFolderMenu::mergeMenus(QDomElement &docElem, QString &name)
 static QString makeRelative(const QString &dir)
 {
     const QString canonical = QDir(dir).canonicalPath();
-    Q_FOREACH (const QString &base, QStandardPaths::locateAll(QStandardPaths::GenericConfigLocation, QLatin1String("menus"), QStandardPaths::LocateDirectory)) {
+    const auto list = QStandardPaths::locateAll(QStandardPaths::GenericConfigLocation, QStringLiteral("menus"), QStandardPaths::LocateDirectory);
+    for (const QString &base : list) {
         if (canonical.startsWith(base)) {
             return canonical.mid(base.length() + 1);
         }
@@ -682,11 +684,12 @@ VFolderMenu::pushDocInfo(const QString &fileName, const QString &baseDir)
     if (m_docInfo.path.isEmpty()) {
         m_docInfo.baseDir.clear();
         m_docInfo.baseName.clear();
-        qDebug() << "Menu" << fileName << "not found.";
+        qCDebug(SYCOCA) << "Menu" << fileName << "not found.";
         return;
     }
+    qCDebug(SYCOCA) << "Found menu file" << m_docInfo.path;
     int i;
-    i = baseName.lastIndexOf('/');
+    i = baseName.lastIndexOf(QLatin1Char('/'));
     if (i > 0) {
         m_docInfo.baseDir = baseName.left(i + 1);
         m_docInfo.baseName = baseName.mid(i + 1, baseName.length() - i - 6);
@@ -703,11 +706,11 @@ VFolderMenu::pushDocInfoParent(const QString &basePath, const QString &baseDir)
 
     m_docInfo.baseDir = baseDir;
 
-    QString fileName = basePath.mid(basePath.lastIndexOf('/') + 1);
+    QString fileName = basePath.mid(basePath.lastIndexOf(QLatin1Char('/')) + 1);
     m_docInfo.baseName = fileName.left(fileName.length() - 5);   // without ".menu"
     QString baseName = QDir::cleanPath(m_docInfo.baseDir + fileName);
 
-    QStringList result = QStandardPaths::locateAll(QStandardPaths::GenericConfigLocation, QStringLiteral("menus/") + baseName);
+    QStringList result = QStandardPaths::locateAll(QStandardPaths::GenericConfigLocation, QLatin1String("menus/") + baseName);
 
     // Remove anything "more local" than basePath.
     while (!result.isEmpty() && (result.at(0) != basePath)) {
@@ -750,13 +753,13 @@ VFolderMenu::locateMenuFile(const QString &fileName)
         }
 
         QString baseName = QDir::cleanPath(m_docInfo.baseDir +
-                                           fileInfo.path() + '/' + fileNameOnly);
-        result = QStandardPaths::locate(QStandardPaths::GenericConfigLocation, QStringLiteral("menus/") + baseName);
+                                           fileInfo.path() + QLatin1Char('/') + fileNameOnly);
+        result = QStandardPaths::locate(QStandardPaths::GenericConfigLocation, QLatin1String("menus/") + baseName);
     }
 
     if (result.isEmpty()) {
         QString baseName = QDir::cleanPath(m_docInfo.baseDir + fileName);
-        result = QStandardPaths::locate(QStandardPaths::GenericConfigLocation, QStringLiteral("menus/") + baseName);
+        result = QStandardPaths::locate(QStandardPaths::GenericConfigLocation, QLatin1String("menus/") + baseName);
     }
 
     return result;
@@ -812,7 +815,7 @@ VFolderMenu::loadMenu(const QString &fileName)
 
     if (m_doc.isNull()) {
         if (m_docInfo.path.isEmpty()) {
-            qCritical() << fileName << " not found in " << m_allDirectories << endl;
+            qCritical() << fileName << " not found in " << m_allDirectories;
         } else {
             qCWarning(SYCOCA) << "Load error (" << m_docInfo.path << ")";
         }
@@ -913,7 +916,7 @@ VFolderMenu::processCondition(QDomElement &domElem, QHash<QString, KService::Ptr
         FOR_ALL_APPLICATIONS_END
     } else if (domElem.tagName() == QLatin1String("Filename")) {
         const QString filename = domElem.text();
-        //qDebug() << "Adding file" << filename;
+        //qCDebug(SYCOCA) << "Adding file" << filename;
         KService::Ptr s = findApplication(filename);
         if (s) {
             items.insert(filename, s);
@@ -924,7 +927,7 @@ VFolderMenu::processCondition(QDomElement &domElem, QHash<QString, KService::Ptr
 void
 VFolderMenu::loadApplications(const QString &dir, const QString &prefix)
 {
-    //qDebug() << "Looking up applications under" << dir;
+    qCDebug(SYCOCA) << "Looking up applications under" << dir;
 
     QDirIterator it(dir);
     while (it.hasNext()) {
@@ -935,7 +938,7 @@ VFolderMenu::loadApplications(const QString &dir, const QString &prefix)
             if (fn == QLatin1String(".") || fn == QLatin1String("..")) {
                 continue;
             }
-            loadApplications(fi.filePath(), prefix + fn + '-');
+            loadApplications(fi.filePath(), prefix + fn + QLatin1Char('-'));
             continue;
         }
         if (fi.isFile()) {
@@ -943,6 +946,11 @@ VFolderMenu::loadApplications(const QString &dir, const QString &prefix)
                 continue;
             }
             KService::Ptr service = m_kbuildsycocaInterface->createService(fi.absoluteFilePath());
+#ifndef NDEBUG
+            if (fn.contains(QLatin1String("fake"))) {
+                qCDebug(SYCOCA) << "createService" << fi.absoluteFilePath() << "returned" << (service ? "valid service" : "NULL SERVICE");
+            }
+#endif
             if (service) {
                 addApplication(prefix + fn, service);
             }
@@ -953,7 +961,7 @@ VFolderMenu::loadApplications(const QString &dir, const QString &prefix)
 void
 VFolderMenu::processLegacyDir(const QString &dir, const QString &relDir, const QString &prefix)
 {
-    //qDebug().nospace() << "processLegacyDir(" << dir << ", " << relDir << ", " << prefix << ")";
+    //qCDebug(SYCOCA).nospace() << "processLegacyDir(" << dir << ", " << relDir << ", " << prefix << ")";
 
     QHash<QString, KService::Ptr> items;
     QDirIterator it(dir);
@@ -969,11 +977,11 @@ VFolderMenu::processLegacyDir(const QString &dir, const QString &relDir, const Q
 
             m_currentMenu = new SubMenu;
             m_currentMenu->name = fn;
-            m_currentMenu->directoryFile = fi.absoluteFilePath() + QStringLiteral("/.directory");
+            m_currentMenu->directoryFile = fi.absoluteFilePath() + QLatin1String("/.directory");
 
             parentMenu->subMenus.append(m_currentMenu);
 
-            processLegacyDir(fi.filePath(), relDir + fn + '/', prefix);
+            processLegacyDir(fi.filePath(), relDir + fn + QLatin1Char('/'), prefix);
             m_currentMenu = parentMenu;
             continue;
         }
@@ -1044,7 +1052,7 @@ VFolderMenu::processMenu(QDomElement &docElem, int pass)
         m_currentMenu = nullptr;
         // Look up menu
         if (parentMenu) {
-            foreach (SubMenu *menu, parentMenu->subMenus) {
+            for (SubMenu *menu : qAsConst(parentMenu->subMenus)) {
                 if (menu->name == name) {
                     m_currentMenu = menu;
                     break;
@@ -1064,7 +1072,7 @@ VFolderMenu::processMenu(QDomElement &docElem, int pass)
             }
         }
         if (directoryFile.isEmpty()) {
-            //qDebug() << "Menu" << name << "does not specify a directory file.";
+            //qCDebug(SYCOCA) << "Menu" << name << "does not specify a directory file.";
         }
 
         // Override previous directoryFile iff available
@@ -1079,7 +1087,7 @@ VFolderMenu::processMenu(QDomElement &docElem, int pass)
     } else {
         // Look up menu
         if (parentMenu) {
-            foreach (SubMenu *menu, parentMenu->subMenus) {
+            for (SubMenu *menu : qAsConst(parentMenu->subMenus)) {
                 if (menu->name == name) {
                     m_currentMenu = menu;
                     break;
@@ -1218,7 +1226,7 @@ VFolderMenu::processMenu(QDomElement &docElem, int pass)
                     }
                     n2 = n2.nextSibling();
                 }
-                //qDebug() << "Moving" << orig << "to" << dest;
+                //qCDebug(SYCOCA) << "Moving" << orig << "to" << dest;
                 if (!orig.isEmpty() && !dest.isEmpty()) {
                     SubMenu *menu = takeSubMenu(m_currentMenu, orig);
                     if (menu) {
@@ -1248,22 +1256,22 @@ static QString parseAttribute(const QDomElement &e)
     if (e.hasAttribute(SHOW_EMPTY)) {
         QString str = e.attribute(SHOW_EMPTY);
         if (str == QLatin1String("true")) {
-            option = "ME ";
+            option = QStringLiteral("ME ");
         } else if (str == QLatin1String("false")) {
-            option = "NME ";
+            option = QStringLiteral("NME ");
         } else {
-            //qDebug()<<" Error in parsing show_empty attribute :"<<str;
+            //qCDebug(SYCOCA)<<" Error in parsing show_empty attribute :"<<str;
         }
     }
     const QString INLINE=QStringLiteral("inline");
     if (e.hasAttribute(INLINE)) {
         QString str = e.attribute(INLINE);
         if (str == QLatin1String("true")) {
-            option += "I ";
+            option += QLatin1String("I ");
         } else if (str == QLatin1String("false")) {
-            option += "NI ";
+            option += QLatin1String("NI ");
         } else {
-            qDebug() << " Error in parsing inline attribute :" << str;
+            qCDebug(SYCOCA) << " Error in parsing inline attribute :" << str;
         }
     }
     if (e.hasAttribute(QStringLiteral("inline_limit"))) {
@@ -1276,26 +1284,26 @@ static QString parseAttribute(const QDomElement &e)
     if (e.hasAttribute(QStringLiteral("inline_header"))) {
         QString str = e.attribute(QStringLiteral("inline_header"));
         if (str == QLatin1String("true")) {
-            option += QStringLiteral("IH ");
+            option += QLatin1String("IH ");
         } else if (str == QLatin1String("false")) {
-            option += QStringLiteral("NIH ");
+            option += QLatin1String("NIH ");
         } else {
-            qDebug() << " Error in parsing of inline_header attribute :" << str;
+            qCDebug(SYCOCA) << " Error in parsing of inline_header attribute :" << str;
         }
 
     }
     if (e.hasAttribute(QStringLiteral("inline_alias")) && e.attribute(QStringLiteral("inline_alias")) == QLatin1String("true")) {
         QString str = e.attribute(QStringLiteral("inline_alias"));
         if (str == QLatin1String("true")) {
-            option += "IA";
+            option += QLatin1String("IA");
         } else if (str == QLatin1String("false")) {
-            option += "NIA";
+            option += QLatin1String("NIA");
         } else {
-            qDebug() << " Error in parsing inline_alias attribute :" << str;
+            qCDebug(SYCOCA) << " Error in parsing inline_alias attribute :" << str;
         }
     }
     if (!option.isEmpty()) {
-        option = option.prepend(QStringLiteral(":O"));
+        option.prepend(QStringLiteral(":O"));
     }
     return option;
 
@@ -1322,7 +1330,7 @@ static QStringList parseLayoutNode(const QDomElement &docElem)
         } else if (e.tagName() == QLatin1String("Filename")) {
             layout.append(e.text());
         } else if (e.tagName() == QLatin1String("Menuname")) {
-            layout.append('/' + e.text());
+            layout.append(QLatin1Char('/') + e.text());
             QString option = parseAttribute(e);
             if (!option.isEmpty()) {
                 layout.append(option);
@@ -1366,7 +1374,7 @@ VFolderMenu::layoutMenu(VFolderMenu::SubMenu *menu, QStringList defaultLayout) /
         }
     }
 
-    foreach (VFolderMenu::SubMenu *subMenu, menu->subMenus) {
+    for (VFolderMenu::SubMenu *subMenu : qAsConst(menu->subMenus)) {
         layoutMenu(subMenu, defaultLayout);
     }
 }
@@ -1374,7 +1382,7 @@ VFolderMenu::layoutMenu(VFolderMenu::SubMenu *menu, QStringList defaultLayout) /
 void
 VFolderMenu::markUsedApplications(const QHash<QString, KService::Ptr> &items)
 {
-    foreach (const KService::Ptr &p, items) {
+    for (const KService::Ptr &p : items) {
         m_usedAppsDict.insert(p->menuId());
     }
 }
